package org.eclipse.jdt.internal.compiler.ast;

import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.core.compiler.CharOperation;
import java.util.Map;
import java.util.HashMap;
import org.eclipse.jdt.internal.compiler.lookup.LocalVariableBinding;
import org.eclipse.jdt.internal.compiler.lookup.PackageBinding;
import org.eclipse.jdt.internal.compiler.lookup.AnnotationBinding;
import org.eclipse.jdt.internal.compiler.lookup.SourceTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.Binding;
import org.eclipse.jdt.internal.compiler.lookup.ParameterizedMethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.InferenceContext18;
import org.eclipse.jdt.internal.compiler.lookup.ProblemMethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeVariableBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.env.AccessRestriction;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.FieldBinding;
import org.eclipse.jdt.internal.compiler.lookup.ArrayBinding;
import org.eclipse.jdt.internal.compiler.lookup.ParameterizedGenericMethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.InvocationSite;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.WildcardBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.TypeIds;
import org.eclipse.jdt.internal.compiler.lookup.TypeConstants;

public abstract class ASTNode implements TypeConstants, TypeIds
{
    public int sourceStart;
    public int sourceEnd;
    public static final int Bit1 = 1;
    public static final int Bit2 = 2;
    public static final int Bit3 = 4;
    public static final int Bit4 = 8;
    public static final int Bit5 = 16;
    public static final int Bit6 = 32;
    public static final int Bit7 = 64;
    public static final int Bit8 = 128;
    public static final int Bit9 = 256;
    public static final int Bit10 = 512;
    public static final int Bit11 = 1024;
    public static final int Bit12 = 2048;
    public static final int Bit13 = 4096;
    public static final int Bit14 = 8192;
    public static final int Bit15 = 16384;
    public static final int Bit16 = 32768;
    public static final int Bit17 = 65536;
    public static final int Bit18 = 131072;
    public static final int Bit19 = 262144;
    public static final int Bit20 = 524288;
    public static final int Bit21 = 1048576;
    public static final int Bit22 = 2097152;
    public static final int Bit23 = 4194304;
    public static final int Bit24 = 8388608;
    public static final int Bit25 = 16777216;
    public static final int Bit26 = 33554432;
    public static final int Bit27 = 67108864;
    public static final int Bit28 = 134217728;
    public static final int Bit29 = 268435456;
    public static final int Bit30 = 536870912;
    public static final int Bit31 = 1073741824;
    public static final int Bit32 = Integer.MIN_VALUE;
    public static final long Bit32L = 2147483648L;
    public static final long Bit33L = 4294967296L;
    public static final long Bit34L = 8589934592L;
    public static final long Bit35L = 17179869184L;
    public static final long Bit36L = 34359738368L;
    public static final long Bit37L = 68719476736L;
    public static final long Bit38L = 137438953472L;
    public static final long Bit39L = 274877906944L;
    public static final long Bit40L = 549755813888L;
    public static final long Bit41L = 1099511627776L;
    public static final long Bit42L = 2199023255552L;
    public static final long Bit43L = 4398046511104L;
    public static final long Bit44L = 8796093022208L;
    public static final long Bit45L = 17592186044416L;
    public static final long Bit46L = 35184372088832L;
    public static final long Bit47L = 70368744177664L;
    public static final long Bit48L = 140737488355328L;
    public static final long Bit49L = 281474976710656L;
    public static final long Bit50L = 562949953421312L;
    public static final long Bit51L = 1125899906842624L;
    public static final long Bit52L = 2251799813685248L;
    public static final long Bit53L = 4503599627370496L;
    public static final long Bit54L = 9007199254740992L;
    public static final long Bit55L = 18014398509481984L;
    public static final long Bit56L = 36028797018963968L;
    public static final long Bit57L = 72057594037927936L;
    public static final long Bit58L = 144115188075855872L;
    public static final long Bit59L = 288230376151711744L;
    public static final long Bit60L = 576460752303423488L;
    public static final long Bit61L = 1152921504606846976L;
    public static final long Bit62L = 2305843009213693952L;
    public static final long Bit63L = 4611686018427387904L;
    public static final long Bit64L = Long.MIN_VALUE;
    public int bits;
    public static final int ReturnTypeIDMASK = 15;
    public static final int OperatorSHIFT = 6;
    public static final int OperatorMASK = 4032;
    public static final int IsReturnedValue = 16;
    public static final int UnnecessaryCast = 16384;
    public static final int DisableUnnecessaryCastCheck = 32;
    public static final int GenerateCheckcast = 64;
    public static final int UnsafeCast = 128;
    public static final int RestrictiveFlagMASK = 7;
    public static final int IsTypeElided = 2;
    public static final int IsArgument = 4;
    public static final int IsLocalDeclarationReachable = 1073741824;
    public static final int IsForeachElementVariable = 16;
    public static final int ShadowsOuterLocal = 2097152;
    public static final int FirstAssignmentToLocal = 8;
    public static final int NeedReceiverGenericCast = 262144;
    public static final int IsImplicitThis = 4;
    public static final int DepthSHIFT = 5;
    public static final int DepthMASK = 8160;
    public static final int IsCapturedOuterLocal = 524288;
    public static final int IsReachable = Integer.MIN_VALUE;
    public static final int LabelUsed = 64;
    public static final int DocumentedFallthrough = 536870912;
    public static final int DocumentedCasesOmitted = 1073741824;
    public static final int IsSubRoutineEscaping = 16384;
    public static final int IsTryBlockExiting = 536870912;
    public static final int ContainsAssertion = 1;
    public static final int IsLocalType = 256;
    public static final int IsAnonymousType = 512;
    public static final int IsMemberType = 1024;
    public static final int HasAbstractMethods = 2048;
    public static final int IsSecondaryType = 4096;
    public static final int HasBeenGenerated = 8192;
    public static final int HasLocalType = 2;
    public static final int HasBeenResolved = 16;
    public static final int ParenthesizedSHIFT = 21;
    public static final int ParenthesizedMASK = 534773760;
    public static final int IgnoreNoEffectAssignCheck = 536870912;
    public static final int IsStrictlyAssigned = 8192;
    public static final int IsCompoundAssigned = 65536;
    public static final int DiscardEnclosingInstance = 8192;
    public static final int Unchecked = 65536;
    public static final int ResolveJavadoc = 65536;
    public static final int IsUsefulEmptyStatement = 1;
    public static final int UndocumentedEmptyBlock = 8;
    public static final int OverridingMethodWithSupercall = 16;
    public static final int CanBeStatic = 256;
    public static final int ErrorInSignature = 32;
    public static final int NeedFreeReturn = 64;
    public static final int IsDefaultConstructor = 128;
    public static final int HasAllMethodBodies = 16;
    public static final int IsImplicitUnit = 1;
    public static final int InsideJavadoc = 32768;
    public static final int SuperAccess = 16384;
    public static final int Empty = 262144;
    public static final int IsElseIfStatement = 536870912;
    public static final int ThenExit = 1073741824;
    public static final int IsElseStatementUnreachable = 128;
    public static final int IsThenStatementUnreachable = 256;
    public static final int IsSuperType = 16;
    public static final int IsVarArgs = 16384;
    public static final int IgnoreRawTypeCheck = 1073741824;
    public static final int IsAnnotationDefaultValue = 1;
    public static final int IsNonNull = 131072;
    public static final int NeededScope = 536870912;
    public static final int OnDemand = 131072;
    public static final int Used = 2;
    public static final int DidResolve = 262144;
    public static final int IsAnySubRoutineEscaping = 536870912;
    public static final int IsSynchronized = 1073741824;
    public static final int BlockExit = 536870912;
    public static final int IsRecovered = 32;
    public static final int HasSyntaxErrors = 524288;
    public static final int INVOCATION_ARGUMENT_OK = 0;
    public static final int INVOCATION_ARGUMENT_UNCHECKED = 1;
    public static final int INVOCATION_ARGUMENT_WILDCARD = 2;
    public static final int HasTypeAnnotations = 1048576;
    public static final int IsUnionType = 536870912;
    public static final int IsDiamond = 524288;
    public static final int InsideExpressionStatement = 16;
    public static final int IsSynthetic = 64;
    public static final int HasFunctionalInterfaceTypes = 2097152;
    public static final Argument[] NO_ARGUMENTS;
    
    static {
        NO_ARGUMENTS = new Argument[0];
    }
    
    public ASTNode() {
        this.bits = Integer.MIN_VALUE;
    }
    
    private static int checkInvocationArgument(BlockScope scope, Expression argument, TypeBinding parameterType, TypeBinding argumentType, TypeBinding originalParameterType) {
        argument.computeConversion(scope, parameterType, argumentType);
        if (argumentType != TypeBinding.NULL && parameterType.kind() == 516) {
            WildcardBinding wildcard = (WildcardBinding)parameterType;
            if (wildcard.boundKind != 2) {
                return 2;
            }
        }
        TypeBinding checkedParameterType = parameterType;
        if (TypeBinding.notEquals(argumentType, checkedParameterType) && argumentType.needsUncheckedConversion(checkedParameterType)) {
            scope.problemReporter().unsafeTypeConversion(argument, argumentType, checkedParameterType);
            return 1;
        }
        return 0;
    }
    
    public static boolean checkInvocationArguments(BlockScope scope, Expression receiver, TypeBinding receiverType, MethodBinding method, Expression[] arguments, TypeBinding[] argumentTypes, boolean argsContainCast, InvocationSite invocationSite) {
        long sourceLevel = scope.compilerOptions().sourceLevel;
        boolean is1_7 = sourceLevel >= 3342336L;
        TypeBinding[] params = method.parameters;
        int paramLength = params.length;
        boolean isRawMemberInvocation = !method.isStatic() && !receiverType.isUnboundWildcard() && method.declaringClass.isRawType() && method.hasSubstitutedParameters();
        boolean uncheckedBoundCheck = (method.tagBits & 0x100L) != 0x0L;
        MethodBinding rawOriginalGenericMethod = null;
        if (!isRawMemberInvocation && method instanceof ParameterizedGenericMethodBinding) {
            ParameterizedGenericMethodBinding paramMethod = (ParameterizedGenericMethodBinding)method;
            if (paramMethod.isRaw && method.hasSubstitutedParameters()) {
                rawOriginalGenericMethod = method.original();
            }
        }
        int invocationStatus = 0;
        if (arguments == null) {
            if (method.isVarargs()) {
                TypeBinding parameterType2 = ((ArrayBinding)params[paramLength - 1]).elementsType();
                if (!parameterType2.isReifiable() && (!is1_7 || (method.tagBits & 0x8000000000000L) == 0x0L)) {
                    scope.problemReporter().unsafeGenericArrayForVarargs(parameterType2, (ASTNode)invocationSite);
                }
            }
        }
        else {
            if (method.isVarargs()) {
                int lastIndex = paramLength - 1;
                for (int j = 0; j < lastIndex; ++j) {
                    TypeBinding originalRawParam2 = (rawOriginalGenericMethod == null) ? null : rawOriginalGenericMethod.parameters[j];
                    invocationStatus |= checkInvocationArgument(scope, arguments[j], params[j], argumentTypes[j], originalRawParam2);
                }
                int argLength = arguments.length;
                if (lastIndex <= argLength) {
                    TypeBinding parameterType = params[lastIndex];
                    TypeBinding originalRawParam3 = null;
                    if (paramLength != argLength || parameterType.dimensions() != argumentTypes[lastIndex].dimensions()) {
                        parameterType = ((ArrayBinding)parameterType).elementsType();
                        if (!parameterType.isReifiable() && (!is1_7 || (method.tagBits & 0x8000000000000L) == 0x0L)) {
                            scope.problemReporter().unsafeGenericArrayForVarargs(parameterType, (ASTNode)invocationSite);
                        }
                        originalRawParam3 = ((rawOriginalGenericMethod == null) ? null : ((ArrayBinding)rawOriginalGenericMethod.parameters[lastIndex]).elementsType());
                    }
                    for (int k = lastIndex; k < argLength; ++k) {
                        invocationStatus |= checkInvocationArgument(scope, arguments[k], parameterType, argumentTypes[k], originalRawParam3);
                    }
                }
                if (paramLength == argLength) {
                    int varargsIndex = paramLength - 1;
                    ArrayBinding varargsType = (ArrayBinding)params[varargsIndex];
                    TypeBinding lastArgType = argumentTypes[varargsIndex];
                    if (lastArgType == TypeBinding.NULL) {
                        if (!varargsType.leafComponentType().isBaseType() || varargsType.dimensions() != 1) {
                            scope.problemReporter().varargsArgumentNeedCast(method, lastArgType, invocationSite);
                        }
                    }
                    else {
                        int dimensions;
                        if (varargsType.dimensions <= (dimensions = lastArgType.dimensions())) {
                            if (lastArgType.leafComponentType().isBaseType()) {
                                --dimensions;
                            }
                            if (varargsType.dimensions < dimensions) {
                                scope.problemReporter().varargsArgumentNeedCast(method, lastArgType, invocationSite);
                            }
                            else if (varargsType.dimensions == dimensions && TypeBinding.notEquals(lastArgType, varargsType) && TypeBinding.notEquals(lastArgType.leafComponentType().erasure(), varargsType.leafComponentType.erasure()) && lastArgType.isCompatibleWith(varargsType.elementsType()) && lastArgType.isCompatibleWith(varargsType)) {
                                scope.problemReporter().varargsArgumentNeedCast(method, lastArgType, invocationSite);
                            }
                        }
                    }
                }
            }
            else {
                for (int i = 0; i < paramLength; ++i) {
                    TypeBinding originalRawParam = (rawOriginalGenericMethod == null) ? null : rawOriginalGenericMethod.parameters[i];
                    invocationStatus |= checkInvocationArgument(scope, arguments[i], params[i], argumentTypes[i], originalRawParam);
                }
            }
            if (argsContainCast) {
                CastExpression.checkNeedForArgumentCasts(scope, receiver, receiverType, method, arguments, argumentTypes, invocationSite);
            }
        }
        if ((invocationStatus & 0x2) != 0x0) {
            scope.problemReporter().wildcardInvocation((ASTNode)invocationSite, receiverType, method, argumentTypes);
        }
        else if (!method.isStatic() && !receiverType.isUnboundWildcard() && method.declaringClass.isRawType() && method.hasSubstitutedParameters()) {
            if (scope.compilerOptions().reportUnavoidableGenericTypeProblems || receiver == null || !receiver.forcedToBeRaw(scope.referenceContext())) {
                scope.problemReporter().unsafeRawInvocation((ASTNode)invocationSite, method);
            }
        }
        else if (rawOriginalGenericMethod != null || uncheckedBoundCheck || (invocationStatus & 0x1) != 0x0) {
            if ((method instanceof ParameterizedGenericMethodBinding)) {
                scope.problemReporter().unsafeRawGenericMethodInvocation((ASTNode)invocationSite, method, argumentTypes);
                return true;
            }
            if (sourceLevel >= 3407872L) {
                return true;
            }
        }
        return false;
    }
    
    public ASTNode concreteStatement() {
        return this;
    }
    
    public final boolean isFieldUseDeprecated(FieldBinding field, Scope scope, int filteredBits) {
        if ((this.bits & 0x8000) == 0x0 && (filteredBits & 0x2000) == 0x0 && field.isOrEnclosedByPrivateType() && !scope.isDefinedInField(field)) {
            if ((filteredBits & 0x10000) != 0x0) {
                FieldBinding original2 = field.original();
                ++original2.compoundUseFlag;
            }
            else {
                FieldBinding original = field.original();
                original.modifiers |= 0x8000000;
            }
        }
        if ((field.modifiers & 0x40000) != 0x0) {
            AccessRestriction restriction = scope.environment().getAccessRestriction(field.declaringClass.erasure());
            if (restriction != null) {
                scope.problemReporter().forbiddenReference(field, this, restriction.classpathEntryType, restriction.classpathEntryName, restriction.getProblemId());
            }
        }
        return field.isViewedAsDeprecated() && !scope.isDefinedInSameUnit(field.declaringClass) && (scope.compilerOptions().reportDeprecationInsideDeprecatedCode || !scope.isInsideDeprecatedCode());
    }
    
    public boolean isImplicitThis() {
        return false;
    }
    
    public boolean receiverIsImplicitThis() {
        return false;
    }
    
    public final boolean isMethodUseDeprecated(MethodBinding method, Scope scope, boolean isExplicitUse) {
        if ((this.bits & 0x8000) == 0x0 && method.isOrEnclosedByPrivateType() && !scope.isDefinedInMethod(method)) {
            MethodBinding original = method.original();
            original.modifiers |= 0x8000000;
        }
        if (isExplicitUse && (method.modifiers & 0x40000) != 0x0) {
            AccessRestriction restriction = scope.environment().getAccessRestriction(method.declaringClass.erasure());
            if (restriction != null) {
                scope.problemReporter().forbiddenReference(method, this, restriction.classpathEntryType, restriction.classpathEntryName, restriction.getProblemId());
            }
        }
        return method.isViewedAsDeprecated() && !scope.isDefinedInSameUnit(method.declaringClass) && (isExplicitUse || (method.modifiers & 0x100000) != 0x0) && (scope.compilerOptions().reportDeprecationInsideDeprecatedCode || !scope.isInsideDeprecatedCode());
    }
    
    public boolean isSuper() {
        return false;
    }
    
    public boolean isQualifiedSuper() {
        return false;
    }
    
    public boolean isThis() {
        return false;
    }
    
    public boolean isUnqualifiedSuper() {
        return false;
    }
    
    public final boolean isTypeUseDeprecated(TypeBinding type, Scope scope) {
        if (type.isArrayType()) {
            type = ((ArrayBinding)type).leafComponentType;
        }
        if (type.isBaseType()) {
            return false;
        }
        ReferenceBinding refType = (ReferenceBinding)type;
        if ((this.bits & 0x8000) == 0x0 && refType instanceof TypeVariableBinding) {
            refType.modifiers |= 0x8000000;
        }
        if ((this.bits & 0x8000) == 0x0 && refType.isOrEnclosedByPrivateType() && !scope.isDefinedInType(refType)) {
            ReferenceBinding referenceBinding = (ReferenceBinding)refType.erasure();
            referenceBinding.modifiers |= 0x8000000;
        }
        if (refType.hasRestrictedAccess()) {
            AccessRestriction restriction = scope.environment().getAccessRestriction(type.erasure());
            if (restriction != null) {
                scope.problemReporter().forbiddenReference(type, this, restriction.classpathEntryType, restriction.classpathEntryName, restriction.getProblemId());
            }
        }
        refType.initializeDeprecatedAnnotationTagBits();
        return refType.isViewedAsDeprecated() && !scope.isDefinedInSameUnit(refType) && (scope.compilerOptions().reportDeprecationInsideDeprecatedCode || !scope.isInsideDeprecatedCode());
    }
    
    public abstract StringBuffer print(int p0, StringBuffer p1);
    
    public static StringBuffer printAnnotations(Annotation[] annotations, StringBuffer output) {
        for (int length = annotations.length, i = 0; i < length; ++i) {
            if (i > 0) {
                output.append(" ");
            }
            Annotation annotation2 = annotations[i];
            if (annotation2 != null) {
                annotation2.print(0, output);
            }
            else {
                output.append('?');
            }
        }
        return output;
    }
    
    public static StringBuffer printIndent(int indent, StringBuffer output) {
        for (int i = indent; i > 0; --i) {
            output.append("  ");
        }
        return output;
    }
    
    public static StringBuffer printModifiers(int modifiers, StringBuffer output) {
        if ((modifiers & 0x1) != 0x0) {
            output.append("public ");
        }
        if ((modifiers & 0x2) != 0x0) {
            output.append("private ");
        }
        if ((modifiers & 0x4) != 0x0) {
            output.append("protected ");
        }
        if ((modifiers & 0x8) != 0x0) {
            output.append("static ");
        }
        if ((modifiers & 0x10) != 0x0) {
            output.append("final ");
        }
        if ((modifiers & 0x20) != 0x0) {
            output.append("synchronized ");
        }
        if ((modifiers & 0x40) != 0x0) {
            output.append("volatile ");
        }
        if ((modifiers & 0x80) != 0x0) {
            output.append("transient ");
        }
        if ((modifiers & 0x100) != 0x0) {
            output.append("native ");
        }
        if ((modifiers & 0x400) != 0x0) {
            output.append("abstract ");
        }
        if ((modifiers & 0x10000) != 0x0) {
            output.append("default ");
            return output;
        }
        return output;
    }
    
    public static void resolvePolyExpressionArguments(Invocation invocation, MethodBinding method, TypeBinding[] argumentTypes, BlockScope scope) {
        MethodBinding candidateMethod = method.isValidBinding() ? method : ((method instanceof ProblemMethodBinding) ? ((ProblemMethodBinding)method).closestMatch : null);
        if (candidateMethod == null) {
            return;
        }
        boolean variableArity = candidateMethod.isVarargs();
        TypeBinding[] parameters = candidateMethod.parameters;
        Expression[] arguments = invocation.arguments();
        if (variableArity && arguments != null && parameters.length == arguments.length && arguments[arguments.length - 1].isCompatibleWith(parameters[parameters.length - 1], scope)) {
            variableArity = false;
        }
        for (int i = 0, length = (arguments == null) ? 0 : arguments.length; i < length; ++i) {
            Expression argument = arguments[i];
            TypeBinding parameterType = InferenceContext18.getParameter(parameters, i, variableArity);
            if (parameterType != null && (argumentTypes[i] != null && argumentTypes[i].isPolyType())) {
                argument.setExpectedType(parameterType);
                TypeBinding updatedArgumentType;
                if ((argument instanceof LambdaExpression)) {
                    LambdaExpression lambda = (LambdaExpression)argument;
                    boolean skipKosherCheck = method.problemId() == 3;
                    updatedArgumentType = lambda.resolveType(scope, skipKosherCheck);
                    if (lambda.isCompatibleWith(parameterType, scope)) {
                        if (lambda.hasErrors()) {
                            continue;
                        }
                    }
                    else {
                        continue;
                    }
                }
                else {
                    updatedArgumentType = argument.resolveType(scope);
                }
                if (updatedArgumentType != null && updatedArgumentType.kind() != 65540) {
                    argumentTypes[i] = updatedArgumentType;
                }
            }
        }
        if ((method instanceof ParameterizedGenericMethodBinding)) {
            InferenceContext18 ic18 = invocation.getInferenceContext((ParameterizedMethodBinding)method);
            if (ic18 != null) {
                ic18.flushBoundOutbox();
            }
        }
        return;
    }
    
    public static void resolveAnnotations(BlockScope scope, Annotation[] sourceAnnotations, Binding recipient) {
        resolveAnnotations(scope, sourceAnnotations, recipient, false);
        if ((recipient instanceof SourceTypeBinding)) {
            ((SourceTypeBinding)recipient).evaluateNullAnnotations();
        }
    }
    
    public static AnnotationBinding[] resolveAnnotations(BlockScope scope, Annotation[] sourceAnnotations, Binding recipient, boolean copySE8AnnotationsToType) {
        AnnotationBinding[] annotations = null;
        int length = (sourceAnnotations == null) ? 0 : sourceAnnotations.length;
        if (recipient != null) {
            switch (recipient.kind()) {
                case 16: {
                    PackageBinding packageBinding = (PackageBinding)recipient;
                    if ((packageBinding.tagBits & 0x200000000L) != 0x0L) {
                        return annotations;
                    }
                    packageBinding.tagBits |= 0x600000000L;
                    break;
                }
                case 4:
                case 2052: {
                    ReferenceBinding type = (ReferenceBinding)recipient;
                    if ((type.tagBits & 0x200000000L) != 0x0L) {
                        return annotations;
                    }
                    type.tagBits |= 0x600000000L;
                    if (length > 0) {
                        annotations = new AnnotationBinding[length];
                        type.setAnnotations(annotations);
                        break;
                    }
                    break;
                }
                case 8: {
                    MethodBinding method = (MethodBinding)recipient;
                    if ((method.tagBits & 0x200000000L) != 0x0L) {
                        return annotations;
                    }
                    method.tagBits |= 0x600000000L;
                    if (length > 0) {
                        annotations = new AnnotationBinding[length];
                        method.setAnnotations(annotations);
                        break;
                    }
                    break;
                }
                case 1: {
                    FieldBinding field = (FieldBinding)recipient;
                    if ((field.tagBits & 0x200000000L) != 0x0L) {
                        return annotations;
                    }
                    field.tagBits |= 0x600000000L;
                    if (length > 0) {
                        annotations = new AnnotationBinding[length];
                        field.setAnnotations(annotations);
                        break;
                    }
                    break;
                }
                case 2: {
                    LocalVariableBinding local = (LocalVariableBinding)recipient;
                    if ((local.tagBits & 0x200000000L) != 0x0L) {
                        return annotations;
                    }
                    local.tagBits |= 0x600000000L;
                    if (length > 0) {
                        annotations = new AnnotationBinding[length];
                        local.setAnnotations(annotations, scope);
                        break;
                    }
                    break;
                }
                case 4100:
                case 16388: {
                    annotations = new AnnotationBinding[length];
                    break;
                }
                default: {
                    return annotations;
                }
            }
        }
        if (sourceAnnotations == null) {
            return annotations;
        }
        int i = 0;
        while (i < length) {
            Annotation annotation = sourceAnnotations[i];
            Binding annotationRecipient = annotation.recipient;
            if (annotationRecipient != null && recipient != null) {
                switch (recipient.kind()) {
                    case 16388: {
                        if (annotations != null) {
                            for (int j = 0; j < length; ++j) {
                                annotations[j] = sourceAnnotations[j].getCompilerAnnotation();
                            }
                            break;
                        }
                        break;
                    }
                    case 1: {
                        FieldBinding field = (FieldBinding)recipient;
                        field.tagBits = ((FieldBinding)annotationRecipient).tagBits;
                        if (annotations != null) {
                            for (int k = 0; k < length; ++k) {
                                Annotation annot = sourceAnnotations[k];
                                annotations[k] = annot.getCompilerAnnotation();
                            }
                            break;
                        }
                        break;
                    }
                    case 2: {
                        LocalVariableBinding local = (LocalVariableBinding)recipient;
                        long otherLocalTagBits = ((LocalVariableBinding)annotationRecipient).tagBits;
                        local.tagBits = otherLocalTagBits;
                        if ((otherLocalTagBits & 0x4000000000000L) == 0x0L) {
                            if (annotations != null) {
                                for (int m = 0; m < length; ++m) {
                                    Annotation annot3 = sourceAnnotations[m];
                                    annotations[m] = annot3.getCompilerAnnotation();
                                }
                                break;
                            }
                            break;
                        }
                        else {
                            if (annotations != null) {
                                LocalDeclaration localDeclaration = local.declaration;
                                int declarationSourceEnd = localDeclaration.declarationSourceEnd;
                                int declarationSourceStart = localDeclaration.declarationSourceStart;
                                for (int l = 0; l < length; ++l) {
                                    Annotation annot2 = sourceAnnotations[l];
                                    AnnotationBinding annotationBinding = annot2.getCompilerAnnotation();
                                    if ((annotations[l] = annotationBinding) != null) {
                                        ReferenceBinding annotationType = annotationBinding.getAnnotationType();
                                        if (annotationType != null && annotationType.id == 49) {
                                            annot2.recordSuppressWarnings(scope, declarationSourceStart, declarationSourceEnd, scope.compilerOptions().suppressWarnings);
                                        }
                                    }
                                }
                                break;
                            }
                            break;
                        }
                    }
                }
                return annotations;
            }
            annotation.recipient = recipient;
            annotation.resolveType(scope);
            if (annotations != null) {
                annotations[i] = annotation.getCompilerAnnotation();
            }
            ++i;
        }
        if (recipient != null && recipient.isTaggedRepeatable()) {
            for (i = 0; i < length; ++i) {
                Annotation annotation = sourceAnnotations[i];
                ReferenceBinding annotationType2 = (annotations[i] != null) ? annotations[i].getAnnotationType() : null;
                if (annotationType2 != null && annotationType2.id == 90) {
                    annotation.checkRepeatableMetaAnnotation(scope);
                }
            }
        }
        if (annotations != null && length > 1) {
            AnnotationBinding[] distinctAnnotations = annotations;
            Map<ReferenceBinding, Object> implicitContainerAnnotations = null;
            for (int i2 = 0; i2 < length; ++i2) {
                AnnotationBinding annotation2 = distinctAnnotations[i2];
                if (annotation2 != null) {
                    ReferenceBinding annotationType3 = annotation2.getAnnotationType();
                    boolean foundDuplicate = false;
                    ContainerAnnotation container = null;
                    for (int m = i2 + 1; m < length; ++m) {
                        AnnotationBinding otherAnnotation = distinctAnnotations[m];
                        if (otherAnnotation != null && TypeBinding.equalsEquals(otherAnnotation.getAnnotationType(), annotationType3)) {
                            if (distinctAnnotations == annotations) {
                                System.arraycopy(distinctAnnotations, 0, distinctAnnotations = new AnnotationBinding[length], 0, length);
                            }
                            distinctAnnotations[m] = null;
                            if (annotationType3.isRepeatableAnnotationType()) {
                                Annotation persistibleAnnotation = sourceAnnotations[i2].getPersistibleAnnotation();
                                if ((persistibleAnnotation instanceof ContainerAnnotation)) {
                                    container = (ContainerAnnotation)persistibleAnnotation;
                                }
                                if (container == null) {
                                    ReferenceBinding containerAnnotationType = annotationType3.containerAnnotationType();
                                    container = new ContainerAnnotation(sourceAnnotations[i2], containerAnnotationType, scope);
                                    if (implicitContainerAnnotations == null) {
                                        implicitContainerAnnotations = new HashMap(3);
                                    }
                                    implicitContainerAnnotations.put(containerAnnotationType, sourceAnnotations[i2]);
                                    Annotation.checkForInstancesOfRepeatableWithRepeatingContainerAnnotation(scope, annotationType3, sourceAnnotations);
                                }
                                container.addContainee(sourceAnnotations[m]);
                            }
                            else {
                                foundDuplicate = true;
                                scope.problemReporter().duplicateAnnotation(sourceAnnotations[m], scope.compilerOptions().sourceLevel);
                            }
                        }
                    }
                    if (container != null) {
                        container.resolveType(scope);
                    }
                    if (foundDuplicate) {
                        scope.problemReporter().duplicateAnnotation(sourceAnnotations[i2], scope.compilerOptions().sourceLevel);
                    }
                }
            }
            if (implicitContainerAnnotations != null) {
                for (int i2 = 0; i2 < length; ++i2) {
                    if (distinctAnnotations[i2] != null) {
                        Annotation annotation3 = sourceAnnotations[i2];
                        ReferenceBinding annotationType3 = distinctAnnotations[i2].getAnnotationType();
                        if (implicitContainerAnnotations.containsKey(annotationType3)) {
                            scope.problemReporter().repeatedAnnotationWithContainer((Annotation)implicitContainerAnnotations.get(annotationType3), annotation3);
                        }
                    }
                }
            }
        }
        if (copySE8AnnotationsToType) {
            copySE8AnnotationsToType(scope, recipient, sourceAnnotations, false);
            return annotations;
        }
        return annotations;
    }
    
    public static TypeBinding resolveAnnotations(BlockScope scope, Annotation[][] sourceAnnotations, TypeBinding type) {
        int levels = (sourceAnnotations == null) ? 0 : sourceAnnotations.length;
        if (type == null || levels == 0) {
            return type;
        }
        AnnotationBinding[][] annotationBindings = new AnnotationBinding[levels][];
        for (int i = 0; i < levels; ++i) {
            Annotation[] annotations = sourceAnnotations[i];
            if (annotations != null && annotations.length > 0) {
                annotationBindings[i] = resolveAnnotations(scope, annotations, TypeBinding.TYPE_USE_BINDING, false);
            }
        }
        return scope.environment().createAnnotatedType(type, annotationBindings);
    }
    
    public static void copySE8AnnotationsToType(BlockScope scope, Binding recipient, Annotation[] annotations, boolean annotatingEnumerator) {
        if (annotations == null || annotations.length == 0 || recipient == null) {
            return;
        }
        long recipientTargetMask = 0L;
        switch (recipient.kind()) {
            case 2: {
                recipientTargetMask = (recipient.isParameter() ? 549755813888L : 2199023255552L);
                break;
            }
            case 1: {
                recipientTargetMask = 137438953472L;
                break;
            }
            case 8: {
                MethodBinding method = (MethodBinding)recipient;
                recipientTargetMask = (method.isConstructor() ? 1099511627776L : 274877906944L);
                break;
            }
            default: {
                return;
            }
        }
        AnnotationBinding[] se8Annotations = null;
        int se8count = 0;
        long se8nullBits = 0L;
        Annotation se8NullAnnotation = null;
        int firstSE8 = -1;
        for (int i = 0, length = annotations.length; i < length; ++i) {
            AnnotationBinding annotation = annotations[i].getCompilerAnnotation();
            if (annotation != null) {
                ReferenceBinding annotationType = annotation.getAnnotationType();
                long metaTagBits = annotationType.getAnnotationTagBits();
                if ((metaTagBits & 0x20000000000000L) != 0x0L) {
                    if (annotatingEnumerator) {
                        if ((metaTagBits & recipientTargetMask) == 0x0L) {
                            scope.problemReporter().misplacedTypeAnnotations(annotations[i], annotations[i]);
                        }
                    }
                    else {
                        if (firstSE8 == -1) {
                            firstSE8 = i;
                        }
                        if (se8Annotations == null) {
                            se8Annotations = new AnnotationBinding[] { annotation };
                            se8count = 1;
                        }
                        else {
                            System.arraycopy(se8Annotations, 0, se8Annotations = new AnnotationBinding[se8count + 1], 0, se8count);
                            se8Annotations[se8count++] = annotation;
                        }
                        if (annotationType.hasNullBit(32)) {
                            se8nullBits |= 0x100000000000000L;
                            se8NullAnnotation = annotations[i];
                        }
                        else if (annotationType.hasNullBit(64)) {
                            se8nullBits |= 0x80000000000000L;
                            se8NullAnnotation = annotations[i];
                        }
                    }
                }
            }
        }
        if (se8Annotations != null) {
            switch (recipient.kind()) {
                case 2: {
                    LocalVariableBinding local = (LocalVariableBinding)recipient;
                    TypeReference typeRef = local.declaration.type;
                    if (Annotation.isTypeUseCompatible(typeRef, scope)) {
                        LocalDeclaration declaration = local.declaration;
                        declaration.bits |= 0x100000;
                        typeRef.bits |= 0x100000;
                        local.type = mergeAnnotationsIntoType(scope, se8Annotations, se8nullBits, se8NullAnnotation, typeRef, local.type);
                        if (scope.environment().usesNullTypeAnnotations()) {
                            local.tagBits &= ~se8nullBits;
                            break;
                        }
                        break;
                    }
                    else {
                        break;
                    }
                }
                case 1: {
                    FieldBinding field = (FieldBinding)recipient;
                    SourceTypeBinding sourceType = (SourceTypeBinding)field.declaringClass;
                    FieldDeclaration fieldDeclaration = sourceType.scope.referenceContext.declarationOf(field);
                    if (Annotation.isTypeUseCompatible(fieldDeclaration.type, scope)) {
                        fieldDeclaration.bits |= 0x100000;
                        TypeReference type = fieldDeclaration.type;
                        type.bits |= 0x100000;
                        field.type = mergeAnnotationsIntoType(scope, se8Annotations, se8nullBits, se8NullAnnotation, fieldDeclaration.type, field.type);
                        if (scope.environment().usesNullTypeAnnotations()) {
                            field.tagBits &= ~se8nullBits;
                            break;
                        }
                        break;
                    }
                    else {
                        break;
                    }
                }
                case 8: {
                    MethodBinding method2 = (MethodBinding)recipient;
                    if (!method2.isConstructor()) {
                        SourceTypeBinding sourceType = (SourceTypeBinding)method2.declaringClass;
                        MethodDeclaration methodDecl = (MethodDeclaration)sourceType.scope.referenceContext.declarationOf(method2);
                        if (Annotation.isTypeUseCompatible(methodDecl.returnType, scope)) {
                            methodDecl.bits |= 0x100000;
                            TypeReference returnType = methodDecl.returnType;
                            returnType.bits |= 0x100000;
                            method2.returnType = mergeAnnotationsIntoType(scope, se8Annotations, se8nullBits, se8NullAnnotation, methodDecl.returnType, method2.returnType);
                            if (scope.environment().usesNullTypeAnnotations()) {
                                method2.tagBits &= ~se8nullBits;
                                break;
                            }
                            break;
                        }
                        else {
                            break;
                        }
                    }
                    else {
                        method2.setTypeAnnotations(se8Annotations);
                        break;
                    }
                }
            }
            AnnotationBinding[] recipientAnnotations = recipient.getAnnotations();
            int length = (recipientAnnotations == null) ? 0 : recipientAnnotations.length;
            int newLength = 0;
            for (AnnotationBinding recipientAnnotation : recipientAnnotations) {
                if (recipientAnnotation != null) {
                    long annotationTargetMask = recipientAnnotation.getAnnotationType().getAnnotationTagBits() & 0x600FF800000000L;
                    if (annotationTargetMask == 0L || (annotationTargetMask & recipientTargetMask) != 0x0L) {
                        recipientAnnotations[newLength++] = recipientAnnotation;
                    }
                }
            }
            if (newLength != length) {
                System.arraycopy(recipientAnnotations, 0, recipientAnnotations = new AnnotationBinding[newLength], 0, newLength);
                recipient.setAnnotations(recipientAnnotations, scope);
            }
        }
        return;
    }
    
    private static TypeBinding mergeAnnotationsIntoType(BlockScope scope, AnnotationBinding[] se8Annotations, long se8nullBits, Annotation se8NullAnnotation, TypeReference typeRef, TypeBinding existingType) {
        if (existingType == null || !existingType.isValidBinding()) {
            return existingType;
        }
        TypeReference unionRef = typeRef.isUnionType() ? ((UnionTypeReference)typeRef).typeReferences[0] : null;
        TypeBinding oldLeafType = (unionRef == null) ? existingType.leafComponentType() : unionRef.resolvedType;
        if (se8nullBits != 0L && oldLeafType.isBaseType()) {
            scope.problemReporter().illegalAnnotationForBaseType(typeRef, new Annotation[] { se8NullAnnotation }, se8nullBits);
            return existingType;
        }
        long prevNullBits = oldLeafType.tagBits & 0x180000000000000L;
        if ((prevNullBits | se8nullBits) == 0x180000000000000L) {
            if (!(oldLeafType instanceof TypeVariableBinding)) {
                if (prevNullBits != 108086391056891904L && se8nullBits != 108086391056891904L) {
                    scope.problemReporter().contradictoryNullAnnotations(se8NullAnnotation);
                }
                se8Annotations = Binding.NO_ANNOTATIONS;
                se8nullBits = 0L;
            }
            oldLeafType = oldLeafType.withoutToplevelNullAnnotation();
        }
        AnnotationBinding[][] goodies = new AnnotationBinding[typeRef.getAnnotatableLevels()][];
        goodies[0] = se8Annotations;
        TypeBinding newLeafType = scope.environment().createAnnotatedType(oldLeafType, goodies);
        if (unionRef == null) {
            typeRef.resolvedType = (existingType.isArrayType() ? scope.environment().createArrayType(newLeafType, existingType.dimensions(), existingType.getTypeAnnotations()) : newLeafType);
        }
        else {
            unionRef.resolvedType = newLeafType;
            unionRef.bits |= 0x100000;
        }
        return typeRef.resolvedType;
    }
    
    public static void resolveDeprecatedAnnotations(BlockScope scope, Annotation[] annotations, Binding recipient) {
        if (recipient != null) {
            int kind = recipient.kind();
            int length;
            if (annotations != null && (length = annotations.length) >= 0) {
                switch (kind) {
                    case 16: {
                        PackageBinding packageBinding = (PackageBinding)recipient;
                        if ((packageBinding.tagBits & 0x400000000L) != 0x0L) {
                            return;
                        }
                        break;
                    }
                    case 4:
                    case 2052: {
                        ReferenceBinding type = (ReferenceBinding)recipient;
                        if ((type.tagBits & 0x400000000L) != 0x0L) {
                            return;
                        }
                        break;
                    }
                    case 8: {
                        MethodBinding method = (MethodBinding)recipient;
                        if ((method.tagBits & 0x400000000L) != 0x0L) {
                            return;
                        }
                        break;
                    }
                    case 1: {
                        FieldBinding field = (FieldBinding)recipient;
                        if ((field.tagBits & 0x400000000L) != 0x0L) {
                            return;
                        }
                        break;
                    }
                    case 2: {
                        LocalVariableBinding local = (LocalVariableBinding)recipient;
                        if ((local.tagBits & 0x400000000L) != 0x0L) {
                            return;
                        }
                        break;
                    }
                    default: {
                        return;
                    }
                }
                for (int i = 0; i < length; ++i) {
                    TypeReference annotationTypeRef = annotations[i].type;
                    if (CharOperation.equals(TypeConstants.JAVA_LANG_DEPRECATED[2], annotationTypeRef.getLastToken())) {
                        TypeBinding annotationType = annotations[i].type.resolveType(scope);
                        if (annotationType != null && annotationType.isValidBinding() && annotationType.id == 44) {
                            switch (kind) {
                                case 16: {
                                    PackageBinding packageBinding2 = (PackageBinding)recipient;
                                    packageBinding2.tagBits |= 0x400400000000L;
                                    return;
                                }
                                case 4:
                                case 2052:
                                case 4100: {
                                    ReferenceBinding type2 = (ReferenceBinding)recipient;
                                    type2.tagBits |= 0x400400000000L;
                                    return;
                                }
                                case 8: {
                                    MethodBinding method2 = (MethodBinding)recipient;
                                    method2.tagBits |= 0x400400000000L;
                                    return;
                                }
                                case 1: {
                                    FieldBinding field2 = (FieldBinding)recipient;
                                    field2.tagBits |= 0x400400000000L;
                                    return;
                                }
                                case 2: {
                                    LocalVariableBinding local2 = (LocalVariableBinding)recipient;
                                    local2.tagBits |= 0x400400000000L;
                                    return;
                                }
                                default: {
                                    return;
                                }
                            }
                        }
                    }
                }
            }
            switch (kind) {
                case 16: {
                    PackageBinding packageBinding3 = (PackageBinding)recipient;
                    packageBinding3.tagBits |= 0x400000000L;
                    return;
                }
                case 4:
                case 2052:
                case 4100: {
                    ReferenceBinding type3 = (ReferenceBinding)recipient;
                    type3.tagBits |= 0x400000000L;
                    return;
                }
                case 8: {
                    MethodBinding method3 = (MethodBinding)recipient;
                    method3.tagBits |= 0x400000000L;
                    return;
                }
                case 1: {
                    FieldBinding field3 = (FieldBinding)recipient;
                    field3.tagBits |= 0x400000000L;
                    return;
                }
                case 2: {
                    LocalVariableBinding local3 = (LocalVariableBinding)recipient;
                    local3.tagBits |= 0x400000000L;
                    return;
                }
                default: {
                    return;
                }
            }
        }
        else {
            return;
        }
    }
    
    public boolean checkingPotentialCompatibility() {
        return false;
    }
    
    public void acceptPotentiallyCompatibleMethods(MethodBinding[] methods) {
    }
    
    public int sourceStart() {
        return this.sourceStart;
    }
    
    public int sourceEnd() {
        return this.sourceEnd;
    }
    
    @Override
    public String toString() {
        return this.print(0, new StringBuffer(30)).toString();
    }
    
    public void traverse(ASTVisitor visitor, BlockScope scope) {
    }
}
